@@ -173,6 +173,36 @@ class PoincareBall(Hyperbolic):
 
         return weighted_pdf
 
+    @geomstats.vectorization.decorator(['else', 'vector'])
+    def projection(self, point):
+        """Project a point on the ball.
+
+        Project a point by clipping such that l2
+        norm being lower than 1
+
+        Parameters
+        ----------
+        point : array-like, shape=[..., dim]
+            Point in embedding Euclidean space.
+
+        Returns
+        -------
+        projected_point : array-like, shape=[..., dim]
+            Point projected on the ball.
+        """
+        if point.shape[-1] != self.dim:
+            raise NameError("Bad dimension expected ", self.dim)
+
+        l2_norm = gs.linalg.norm(point, axis=-1)
+        if gs.any(l2_norm >= 1 - EPSILON):
+            projected_point =\
+                gs.einsum('...j,...->...j', point * (1 - EPSILON - TOLERANCE),
+                          1. / l2_norm)
+            projected_point = -gs.maximum(-projected_point, -point)
+            return projected_point
+
+        return point
+
 
 class PoincareBallMetric(RiemannianMetric):
     """Class that defines operations using a Poincare ball.
@@ -235,10 +265,6 @@ class PoincareBallMetric(RiemannianMetric):
             base_point,
             gs.einsum('...i,...->...i', direction, factor))
 
-        if gs.any(zero_tan):
-            exp = gs.assignment(
-                exp, base_point[zero_tan], zero_tan)
-
         return exp
 
     @geomstats.vectorization.decorator(['else', 'vector', 'vector'])
@@ -288,8 +314,8 @@ class PoincareBallMetric(RiemannianMetric):
             '...i,...j->...j', log, add_base_point)
         return log
 
-    @geomstats.vectorization.decorator(['else', 'vector', 'vector'])
-    def mobius_add(self, point_a, point_b):
+    @geomstats.vectorization.decorator(['else', 'vector', 'vector', 'else'])
+    def mobius_add(self, point_a, point_b, auto_project=True):
         r"""Compute the Mobius addition of two points.
 
         Mobius addition operation that is a necessary operation
@@ -303,9 +329,11 @@ class PoincareBallMetric(RiemannianMetric):
         Parameters
         ----------
         point_a : array-like, shape=[..., dim]
-            Point in hyperbolic space.
+            Point in Poincare ball associated space.
         point_b : array-like, shape=[..., dim]
-            Point in hyperbolic space.
+            Point in Poincare ball associated space.
+        auto_project : boolean
+            Project points on the ball or not (according to tolerance).
 
         Returns
         -------
@@ -313,11 +341,15 @@ class PoincareBallMetric(RiemannianMetric):
             Result of the Mobius addition.
         """
         ball_manifold = PoincareBall(self.dim, scale=self.scale)
-        point_a_belong = ball_manifold.belongs(point_a)
-        point_b_belong = ball_manifold.belongs(point_b)
+        if auto_project:
+            point_a = ball_manifold.projection(point_a)
+            point_b = ball_manifold.projection(point_b)
+        else:
+            point_a_belong = ball_manifold.belongs(point_a)
+            point_b_belong = ball_manifold.belongs(point_b)
 
-        if (not gs.all(point_a_belong) or not gs.all(point_b_belong)):
-            raise ValueError("Points do not belong to the Poincare ball")
+            if (not gs.all(point_a_belong) or not gs.all(point_b_belong)):
+                raise ValueError("Points do not belong to the Poincare ball")
 
         norm_point_a = gs.sum(point_a ** 2, axis=-1, keepdims=True)
         norm_point_b = gs.sum(point_b ** 2, axis=-1, keepdims=True)
@@ -334,8 +366,7 @@ class PoincareBallMetric(RiemannianMetric):
 
         mobius_add = gs.einsum(
             '...i,...k->...i', add_nominator, 1 / add_denominator)
-
-        return mobius_add
+        return ball_manifold.projection(mobius_add)
 
     @geomstats.vectorization.decorator(['else', 'vector', 'vector'])
     def dist_broadcast(self, point_a, point_b):
@@ -456,7 +487,7 @@ class PoincareBallMetric(RiemannianMetric):
             - gs.einsum('...i,...j->...j', retraction_factor, tangent_vec)
 
     @geomstats.vectorization.decorator(['else', 'vector'])
-    def inner_product_matrix(self, base_point=None):
+    def metric_matrix(self, base_point=None):
         """Compute the inner product matrix.
 
         Parameters
