@@ -8,9 +8,11 @@ import warnings
 
 import numpy as _np
 import scipy.linalg
+import torch
 
 import geomstats.backend as gs
 import geomstats.tests
+from geomstats.geometry.spd_matrices import SPDMatrices
 from geomstats.geometry.special_orthogonal import SpecialOrthogonal
 
 
@@ -183,46 +185,6 @@ class TestBackends(geomstats.tests.TestCase):
 
         self.assertAllClose(result, expected)
 
-    @geomstats.tests.np_and_tf_only
-    def test_powerm_diagonal(self):
-        power = .5
-        point = gs.array([[1., 0., 0.],
-                          [0., 4., 0.],
-                          [0., 0., 9.]])
-        result = gs.linalg.powerm(point, power)
-        expected = gs.array([[1., 0., 0.],
-                             [0., 2., 0.],
-                             [0., 0., 3.]])
-
-        self.assertAllClose(result, expected)
-
-    @geomstats.tests.np_and_tf_only
-    def test_powerm(self):
-        power = 2.4
-        point = gs.array([[1., 0., 0.],
-                          [0., 2.5, 1.5],
-                          [0., 1.5, 2.5]])
-        result = gs.linalg.powerm(point, power)
-        result = gs.linalg.powerm(result, 1 / power)
-        expected = point
-
-        self.assertAllClose(result, expected)
-
-    @geomstats.tests.np_only
-    def test_powerm_vectorization(self):
-        power = 2.4
-        points = gs.array([[[1., 0., 0.],
-                            [0., 4., 0.],
-                            [0., 0., 9.]],
-                           [[1., 0., 0.],
-                            [0., 2.5, 1.5],
-                            [0., 1.5, 2.5]]])
-        result = gs.linalg.powerm(points, power)
-        result = gs.linalg.powerm(result, 1. / power)
-        expected = points
-
-        self.assertAllClose(result, expected)
-
     @geomstats.tests.tf_only
     def test_vstack(self):
         import tensorflow as tf
@@ -896,3 +858,145 @@ class TestBackends(geomstats.tests.TestCase):
 
         with self.assertRaises((ValueError, RuntimeError)):
             gs.broadcast_arrays(gs.array([1, 2]), gs.array([3, 4, 5]))
+
+    def test_value_and_grad(self):
+        n = 10
+        vector = gs.ones(n)
+        result_loss, result_grad = gs.autograd.value_and_grad(
+            lambda v: gs.sum(v ** 2))(vector)
+        expected_loss = n
+        expected_grad = 2 * vector
+        self.assertAllClose(result_loss, expected_loss)
+        self.assertAllClose(result_grad, expected_grad)
+
+    def test_value_and_grad_numpy_input(self):
+        n = 10
+        vector = _np.ones(n)
+        result_loss, result_grad = gs.autograd.value_and_grad(
+            lambda v: gs.sum(v ** 2))(vector)
+        expected_loss = n
+        expected_grad = 2 * vector
+        self.assertAllClose(result_loss, expected_loss)
+        self.assertAllClose(result_grad, expected_grad)
+
+    def test_choice(self):
+        x = gs.array([0.1, 0.2, 0.3, 0.4, 0.5])
+        a = 4
+        result = gs.random.choice(x, a)
+
+        result_bool = True
+        for i in result:
+            if i in x:
+                continue
+            result_bool = False
+
+        self.assertTrue(result_bool)
+        self.assertEqual(len(result), a)
+
+    def test_split(self):
+        x = gs.array([0.1, 0.2, 0.3, 0.4])
+        result = gs.split(x, 2)
+        expected = _np.split(x, 2)
+        for res, exp in zip(result, expected):
+            self.assertAllClose(res, exp)
+
+    def test_svd(self):
+        gs_point = gs.reshape(gs.arange(12), (4, 3))
+        gs_point = gs.cast(gs_point, gs.float64)
+        np_point = _np.arange(12).reshape(4, 3)
+        reconstruction = gs.array([
+            [1., 0., 0.], [0., 1., 0.], [0., 0., 1.], [0., 0., 0.]])
+        u, s, v = _np.linalg.svd(np_point)
+        u_r, s_r, v_r = gs.linalg.svd(gs_point)
+        s_r_reconstructed = gs.einsum('kl,l->kl', reconstruction, s_r)
+        gs_a_approx = gs.matmul(gs.matmul(
+            u_r, s_r_reconstructed), v_r)
+        s_reconstructed = _np.einsum('kl,l->kl', reconstruction, s)
+        np_a_approx = _np.dot(u, _np.dot(s_reconstructed, v))
+        self.assertAllClose(gs_a_approx, np_a_approx)
+
+        full_matrices = False
+        u, s, v = _np.linalg.svd(
+            np_point, full_matrices=full_matrices)
+        u_r, s_r, v_r = gs.linalg.svd(
+            gs_point, full_matrices)
+        reconstruction = gs.eye(3)
+        s_r_reconstructed = gs.einsum('kl,l->kl', reconstruction, s_r)
+        gs_a_approx = gs.matmul(gs.matmul(
+            u_r, s_r_reconstructed), v_r)
+        s_reconstructed = _np.einsum('kl,l->kl', reconstruction, s)
+        np_a_approx = _np.dot(u, _np.dot(s_reconstructed, v))
+        self.assertAllClose(gs_a_approx, np_a_approx)
+
+        compute_uv = False
+        s = _np.linalg.svd(np_point, compute_uv=compute_uv)
+        s_r = gs.linalg.svd(gs_point, compute_uv=compute_uv)
+        self.assertAllClose(s, s_r)
+
+    def test_sylvester_solve(self):
+        mat = gs.random.rand(4, 3)
+        spd = gs.matmul(gs.transpose(mat), mat)
+
+        mat = gs.random.rand(3, 3)
+        skew = mat - gs.transpose(mat)
+        solution = gs.linalg.solve_sylvester(spd, spd, skew)
+        result = gs.matmul(spd, solution)
+        result += gs.matmul(solution, spd)
+
+        self.assertAllClose(result, skew)
+
+    @geomstats.tests.np_and_pytorch_only
+    def test_general_sylvester_solve(self):
+        a = gs.array([[-3., -2., 0.], [-1., -1., 3.], [3., -5., -1.]])
+        b = gs.array([[1.]])
+        q = gs.array([[1.], [2.], [3.]])
+        sol = gs.linalg.solve_sylvester(a, b, q)
+        result = gs.matmul(a, sol) + gs.matmul(sol, b)
+        self.assertAllClose(result, q)
+
+    def test_sylvester_solve_vectorization(self):
+        gs.random.seed(0)
+        mat = gs.random.rand(2, 4, 3)
+        spd = gs.matmul(gs.transpose(mat, (0, 2, 1)), mat)
+
+        mat = gs.random.rand(2, 3, 3)
+        skew = mat - gs.transpose(mat, (0, 2, 1))
+        solution = gs.linalg.solve_sylvester(spd, spd, skew)
+        result = gs.matmul(spd, solution)
+        result += gs.matmul(solution, spd)
+
+        self.assertAllClose(result, skew)
+
+    def test_eigvalsh(self):
+        mat = gs.array([[2., 1.], [1., -1.]])
+        result = gs.linalg.eigvalsh(mat, UPLO='U')
+        expected = _np.linalg.eigvalsh(mat)
+        self.assertAllCloseToNp(result, expected)
+
+    def test_cholesky(self):
+        mat = SPDMatrices(3).random_uniform(2)
+        result = gs.linalg.cholesky(mat)
+        expected = _np.linalg.cholesky(mat)
+        self.assertAllClose(result, expected)
+
+    @geomstats.tests.np_and_pytorch_only
+    def test_expm_backward(self):
+        mat = gs.array([[0, 1, .5], [-1, 0, 0.2], [-.5, -.2, 0]])
+
+        def loss(p):
+            return gs.sum((gs.linalg.expm(p) - gs.eye(3)) ** 2)
+
+        value_and_grad = gs.autograd.value_and_grad(loss)
+        result = value_and_grad(mat)
+
+        def loss_torch(p):
+            return torch.sum((torch.matrix_exp(p) - torch.eye(3)) ** 2)
+
+        torch_mat = torch.tensor(
+            [[0, 1, .5], [-1, 0, 0.2], [-.5, -.2, 0]], requires_grad=True)
+        value = loss_torch(torch_mat)
+        value.backward()
+        grad = torch_mat.grad
+
+        self.assertAllClose(result[0], value.detach())
+        self.assertAllClose(result[1], grad)
